#ifndef DYNAMIC_CANVAS_H
#define DYNAMIC_CANVAS_H

#include <vector>

// Adapt for your own viewer camera type.
// Position, orientation, fieldOfView and aspectRatio of the camera are needed.
#include "QGLViewer/camera.h"
#include "QGLViewer/quaternion.h"
#include "QGLViewer/vec.h"

class DynamicCanvas
{
public:
  DynamicCanvas();
  ~DynamicCanvas();

  //! Must be called when you initialize OpenGL. Parameters are the
  //! window dimensions. They must not changed during the execution.
  void initGL(const unsigned int width,
	      const unsigned int height,
	      const qglviewer::Camera& camera);
  
  //! Load a new paper texture image. Returns \p true in case of success.
  bool loadPaperTexture(const char* name);

  //! Interface function. Get all the needed parameters.
  //! Must be called before drawPaper().
  //! Most probably, you need to adapt this function to your own code.
  void updateParameters(const qglviewer::Camera& camera);

  //! Main function, should be called by your drawing function.
  void drawPaper();
  
  //! Whether or not the paper has spherical warp.
  //! Default value is \p true.
  bool isWarped() { return warped;  }
  void setWarped(const bool w) { warped=w; }

  //! Texture scaling factor. Default value is 1.
  float frequencyBase() { return frequencyBase_; }
  void setFrequencyBase(float fb) { frequencyBase_=fb; }
  
  //! The distance for which paper and 3D object displacements
  //! match when zooming forward.
  float subjectiveDistance() { return D; }
  void setSubjectiveDistance(float sd) { D=sd; }

  void makePaperMoreTransparent() { paperTransparency *= 1.2; }
  void makePaperLessTransparent() { paperTransparency /= 1.2; }
  
private:
  // We need a vector type with the common operators.
  typedef qglviewer::Vec vectorType;
  
  /*!
    Make the display list for the grid supporting the paper.
  */
  void buildDisplayList();

  /*!
    Create the multi-scale paper texture.
    To be called first in your draw() method.
    May change some states (GL_DEPTH, GL_COLOR_MATERIAL,..).
  */
  void buildMultiScaleTexture();
  
  /*!
    Set the texture matrix.
  */
  void setTextureMatrix();

  /*!
    Send the warped texture coordinates to OpenGL.
    The screen is coordinated by (-1;+1)x(-1;+1).
  */
  void warpedTextureCoordinate(const float x, const float y);

  /*!
    Send the unwarped texture coordinates to OpenGL.
    The screen is coordinated by (-1;+1)x(-1;+1).
  */
  void unwarpedTextureCoordinate(const float x, const float y);

  /*! Call either warpedTextureCoordinate() or unwarpedTextureCoordinate(). */  
  void textureCoordinate(const float x, const float y);

  unsigned int nbOctaves() { return nbOctaves_; }
  
  //! Choose between warped and unwarped paper.
  bool warped;

  //! Texture scaling factor. See section 5.1.
  float frequencyBase_;

  //! "Hardness" of the paper. Control the blending between the paper
  //! texture and the 3D scene. See section 5.1.
  float paperTransparency;

  //! Subjective distance at which the paper "appears". See Section 3.
  float D;

  //! Texture parameters. See Appendices A and B.
  float posX, posY;
  float ratio;
  float fieldOfView;
  float alpha, beta, gamma;
  float deltaAlpha, deltaBeta, deltaGamma;

  //! Previous camera position.
  vectorType previousCameraPosition;

  //! Radius of the sphere (distance from camezr to texture)
  float sphereRadius;
  
  
  //! OpenGL ID for the display list.
  GLuint paperGridID;

  //! OpenGL ID for the multi-scale paper texture.
  GLuint msTextureID;

  //! OpenGL ID for the base paper texture.
  GLuint paperTextureID;

  //! Index of the first visible octave.
  unsigned int omega0;

  //! Number of octaves in the multi-scale paper texture.
  const unsigned int nbOctaves_;

  //! Size (in pixels) of the multi-scale paper texture.
  unsigned int textureSize_;

  //! Array containing random translation values.
  std::vector<float> randomOffsetX;
  std::vector<float> randomOffsetY;
  
  //! Number of random values used to randomize the position of each octave.
  const unsigned int nbOffset;

  //! alpha and beta (used to shift the texture)
  // are measured with respect to this orientation
  qglviewer::Quaternion initialCameraOrientation_;
  
  //! Zoom factor used to scale the octaves.
  float zoom;
};

#endif // DYNAMIC_CANVAS_H




